﻿using System;
using System.Web;
using System.Collections.Generic;

using CMS.Ecommerce;
using CMS.GlobalHelper;
using CMS.SettingsProvider;
using CMS.SiteProvider;

/// <summary>
/// Sample shopping cart info provider. 
/// Can be registered either by replacing the SKUInfoProvider.ProviderObject (uncomment the line in SampleECommerceModule.cs) or through cms.extensibility section of the web.config
/// </summary>
public class CustomSKUInfoProvider : SKUInfoProvider
{
    #region "Example: Custom SKU base price: 1) SKU price according to the culture 2) SKU price according to the SKU status"

    /// <summary>
    /// Returns catalogue price of the given product based on the SKU data and shopping cart data. 
    /// By default price is loaded from the SKUPrice field.
    /// </summary>
    /// <param name="sku">SKU dta</param>
    /// <param name="cart">Shopping cart data</param>
    protected override double GetSKUPriceInternal(SKUInfo sku, ShoppingCartInfo cart)
    {
        double price = 0;

        // 1) Get base SKU price according to the shopping cart data (current culture)
        if (cart != null)
        {
            switch (cart.ShoppingCartCulture.ToLower())
            {
                case "en-us":
                    // Get price form SKUEnUsPrice field
                    price = ValidationHelper.GetDouble(sku.GetValue("SKUEnUsPrice"), 0);
                    break;

                case "cs-cz":
                    // Get price form SKUCsCzPrice field
                    price = ValidationHelper.GetDouble(sku.GetValue("SKUCsCzPrice"), 0);
                    break;
            }
        }

        //// 2) Get base SKU price according to the product properties (product status)
        //PublicStatusInfo status = PublicStatusInfoProvider.GetPublicStatusInfo(sku.SKUPublicStatusID);
        //if ((status != null) && (status.PublicStatusName.ToLowerCSafe() == "discounted"))
        //{
        //    // Get price form SKUDiscountedPrice field
        //    price = ValidationHelper.GetDouble(sku.GetValue("SKUDiscountedPrice"), 0);
        //}

        if (price == 0)
        {
            // Get price from the SKUPrice field by default
            return base.GetSKUPriceInternal(sku, cart);
        }
        else
        {
            // Return custom price
            return price;
        }
    }

    #endregion


    #region "Example: Custom SKU discount for the users in a specific role"

    /// <summary>
    /// Returns list of catalogue discounts which should be applied to the given product based on the product data and shopping cart data.
    /// </summary>
    /// <param name="sku">SKU data</param>
    /// <param name="cart">Shopping cart data</param>
    /// <param name="forCart">True - This method is called within the shopping cart calculations, False - This method is called within the displaying price on the live site.</param>
    protected override List<IItemDiscount> GetSKUDiscountsInternal(SKUInfo sku, ShoppingCartInfo cart, bool forCart)
    {
        // SKU number of product A
        string skuA = "A";

        // Get SKU default discounts
        List<IItemDiscount> discounts = base.GetSKUDiscountsInternal(sku, cart, forCart);

        // Add extra discount to product A 
        if (sku.SKUNumber.ToLower() == skuA.ToLower())
        {
            // Get user information
            UserInfo user = cart.UserInfoObj;

            // Add discount if user is in VIP role
            if ((user != null) && user.IsInRole("VIP", cart.SiteName))
            {                
                AddDiscountForVIPRole(sku, discounts);
            }
        }

        return discounts;
    }


    /// <summary>
    /// Adds 10% discount to the given sku.
    /// </summary>
    /// <param name="sku">SKU data</param>
    /// <param name="discounts">Discounts of the sku</param>
    private void AddDiscountForVIPRole(SKUInfo sku, List<IItemDiscount> discounts)
    {
        // Create 10% discount
        ItemDiscount discount = new ItemDiscount()
        {
            ItemDiscountID = "DISCOUNT_VIP",
            ItemDiscountDisplayName = "Discount for VIP customers",
            ItemDiscountValue = 10,                
        };

        // Add custom discount to discounts which will be applied to the product
        discounts.Add(discount);        
    }

    #endregion
}