using System;
using System.Web;
using System.Data;

using CMS.Ecommerce;
using CMS.SettingsProvider;
using CMS.SiteProvider;
using CMS.GlobalHelper;

/// <summary>
/// Sample shopping cart info provider. 
/// Can be registered either by replacing the ShoppingCartInfoProvider.ProviderObject (uncomment the line in SampleECommerceModule.cs) or through cms.extensibility section of the web.config
/// </summary>
public class CustomShoppingCartInfoProvider : ShoppingCartInfoProvider
{
    #region "Example: Custom discount applied to the order total price"

    /// <summary>
    /// Calculates discount which should be applied to the total items price.
    /// </summary>
    /// <param name="cart">Shopping cart</param>        
    protected override double CalculateOrderDiscountInternal(ShoppingCartInfo cart)
    {
        // Default order discount
        double result = base.CalculateOrderDiscountInternal(cart);

        // Order total price
        double totalPrice = cart.TotalItemsPriceInMainCurrency;

        // Example of order discount based on the time of shopping - Happy hours (4 PM - 7 PM)
        if ((DateTime.Now.Hour >= 16) && (DateTime.Now.Hour <= 19))
        {
            // 20% discount 
            result = result + totalPrice * 0.2;
        }
        // Example of order discount based on the total price of all cart items
        else if (totalPrice > 500)
        {
            // 10% discount
            result = result + totalPrice * 0.1;
        }
        // Example of order discount based on the discount coupon
        // Please note: You will need to modify ShoppingCartContent.ascx.cs to skip coupon validation for the products
        else if ((cart.DiscountCouponInfoObj != null) && (cart.DiscountCouponInfoObj.IsValid))
        {
            result = result + ECommerceHelper.GetDiscountValue(totalPrice, cart.DiscountCouponInfoObj.DiscountCouponValue, cart.DiscountCouponInfoObj.DiscountCouponIsFlatValue);
        }

        return result;
    }

    #endregion


    #region "Example: Add product A to the shopping cart -> product B is also added (automaticaly)"

    /// <summary>
    /// Adds new item to the shopping cart object and returns its object.
    /// </summary>
    /// <param name="cart">Shopping cart</param>
    /// <param name="itemParams">Parameters from which the new shopping cart item is initialized.</param>
    protected override ShoppingCartItemInfo AddShoppingCartItemInternal(ShoppingCartInfo cart, ShoppingCartItemParameters itemParams)
    {
        // Add item to the shopping cart
        ShoppingCartItemInfo item = base.AddShoppingCartItemInternal(cart, itemParams);

        // SKU number of product A
        string skuA = "A";

        // SKU number of product B
        string skuB = "B";

        // Add product B to the shopping cart automatically after the product A is added
        if (item.SKUObj.SKUNumber.ToLower() == skuA.ToLower())
        {
            string where = string.Format("SKUNumber = N'{0}'", SqlHelperClass.GetSafeQueryString(skuB));
            DataSet products = SKUInfoProvider.GetSKUs(where, null);
            if (!DataHelper.DataSourceIsEmpty(products))
            {
                // Get data of the product B
                SKUInfo productB = new SKUInfo(products.Tables[0].Rows[0]);

                // Configure product B
                ShoppingCartItemParameters itemBParams = new ShoppingCartItemParameters(productB.SKUID, itemParams.Quantity);

                // Add/update product B in the shopping cart
                SetShoppingCartItem(cart, itemBParams);
            }
        }

        return item;
    }

    #endregion
}